<?php
/*
Plugin Name: Plugin Updater Checker
Description: Checks for plugin updates on a remote server and sends the information to a specified endpoint.
Version: 1.0
Author: Nick Murray - Mountain Air Web ltd.
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Hook to add admin menu
add_action('admin_menu', 'plugin_updater_sender_add_admin_menu');

// Hook to register settings
add_action('admin_init', 'plugin_updater_sender_register_settings');

function plugin_updater_sender_add_admin_menu() {
    add_options_page(
        'Plugin Updater Sender',
        'Plugin Updater Sender',
        'manage_options',
        'plugin-updater-sender',
        'plugin_updater_sender_settings_page'
    );
}

function plugin_updater_sender_register_settings() {
    register_setting('plugin_updater_sender_settings', 'plugin_updater_sender_sites');
    // register_setting('plugin_updater_sender_settings', 'plugin_updater_sender_site_url');
    // register_setting('plugin_updater_sender_settings', 'plugin_updater_sender_username');
    // register_setting('plugin_updater_sender_settings', 'plugin_updater_sender_app_password');
}



function plugin_updater_sender_settings_page() {
    ?>
    <div class="wrap">
        <h1>Plugin Updater Checker</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('plugin_updater_sender_settings');
            do_settings_sections('plugin_updater_sender_settings');
            $sites = get_option('plugin_updater_sender_sites', []);
            ?>
            <h2>Sites</h2>
            <table class="form-table" id="sites-table">
                <thead>
                    <tr>
                        <th>Remote Site URL</th>
                        <th>Username</th>
                        <th>Application Password</th>
                        <th>Actions</th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($sites as $index => $site): ?>
                        <tr>
                            <td><input type="text" name="plugin_updater_sender_sites[<?php echo $index; ?>][url]" value="<?php echo esc_attr($site['url']); ?>" size="50"></td>
                            <td><input type="text" name="plugin_updater_sender_sites[<?php echo $index; ?>][username]" value="<?php echo esc_attr($site['username']); ?>" size="20"></td>
                            <td><input type="password" name="plugin_updater_sender_sites[<?php echo $index; ?>][password]" value="<?php echo esc_attr($site['password']); ?>" size="20"></td>
                            <td><button type="button" class="button remove-site">Remove</button></td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>
            <button type="button" class="button" id="add-site">Add Site</button>
            <?php submit_button(); ?>
        </form>
        <form method="post" action="">
            <input type="submit" name="check_updates" class="button button-primary" value="Check for Plugin Updates">
        </form>
        <?php
        if (isset($_POST['check_updates'])) {
            $result = plugin_updater_sender_check_updates();
            if (isset($result['message'])) {
                echo '<div class="notice notice-' . $result['type'] . '"><p>' . $result['message'] . '</p></div>';
            } elseif (isset($result['updates'])) {
                echo '<h2>Available Updates:</h2>';
                foreach ($result['updates'] as $site_url => $updates) {
                    echo '<h3>Site: ' . esc_html($site_url) . '</h3>';
                    echo '<table class="wp-list-table widefat fixed striped">';
                    echo '<thead><tr><th>Plugin</th><th>Current Version</th><th>New Version</th><th>View Details</th><th>Download</th></tr></thead>';
                    echo '<tbody>';
                    foreach ($updates as $update) {
                        echo '<tr>';
                        echo '<td>' . esc_html($update['plugin']) . '</td>';
                        echo '<td>' . esc_html($update['current_version']) . '</td>';
                        echo '<td>' . esc_html($update['new_version']) . '</td>';
                        echo '<td><a href="' . esc_url($update['url']) . '" target="_blank">View Details</a></td>';
                        echo '<td><a href="' . esc_url($update['package']) . '" target="_blank">Download</a></td>';
                        echo '</tr>';
                    }
                    echo '</tbody>';
                    echo '</table>';
                }
                
                // Add a button to send updates to the endpoint
                echo '<form method="post" action="">';
                echo '<input type="hidden" name="send_updates" value="' . esc_attr(json_encode($result['updates'])) . '">';
                // echo '<input type="submit" class="button button-primary" value="Send Updates to Endpoint">';
                echo '</form>';
            }
        }

        // Handle sending updates to the endpoint
        if (isset($_POST['send_updates'])) {
            
            $updates = json_decode(stripslashes($_POST['send_updates']), true);
            $endpoint_url = get_option('plugin_updater_sender_endpoint_url');
            
            $response = wp_remote_post($endpoint_url, [
                'body' => json_encode($updates),
                'headers' => [
                    'Content-Type' => 'application/json',
                ],
            ]);

            if (is_wp_error($response)) {
                echo '<div class="notice notice-error"><p>Failed to send update information to the remote server.</p></div>';
            } else {
                echo '<div class="notice notice-success"><p>Update information sent successfully to the remote server.</p></div>';
            }
        }
        ?>
    </div>
    <script>
        document.getElementById('add-site').addEventListener('click', function() {
            var table = document.getElementById('sites-table').getElementsByTagName('tbody')[0];
            var rowCount = table.rows.length;
            var row = table.insertRow(rowCount);
            row.innerHTML = `
                <td><input type="text" name="plugin_updater_sender_sites[${rowCount}][url]" size="50"></td>
                <td><input type="text" name="plugin_updater_sender_sites[${rowCount}][username]" size="20"></td>
                <td><input type="password" name="plugin_updater_sender_sites[${rowCount}][password]" size="20"></td>
                <td><button type="button" class="button remove-site">Remove</button></td>
            `;
        });

        document.getElementById('sites-table').addEventListener('click', function(e) {
            if (e.target && e.target.classList.contains('remove-site')) {
                var row = e.target.closest('tr');
                row.parentNode.removeChild(row);
            }
        });
    </script>
    <?php
}




function plugin_updater_sender_check_updates() {
    $sites = get_option('plugin_updater_sender_sites', []);
    $all_updates = [];

    foreach ($sites as $site) {
        $site_url = $site['url'];
        $username = $site['username'];
        $application_password = $site['password'];

        $endpoint_url = trailingslashit($site_url) . 'wp-json/plugin-updater/v1/updates';
        $remote_plugin_list_url = trailingslashit($site_url) . 'wp-json/wp/v2/plugins/';

        if (empty($endpoint_url) || empty($remote_plugin_list_url)) {
            echo '<div class="notice notice-error"><p>Please set both the Endpoint URL and Remote Plugin List URL in the settings.</p></div>';
            continue;
        }

        // Base64 encode the username and application password
        $auth = base64_encode($username . ':' . $application_password);

        // Fetch the plugin list from the remote server with authentication
        $response = wp_remote_get($remote_plugin_list_url, [
            'headers' => [
                'Authorization' => 'Basic ' . $auth,
            ],
        ]);

        if (is_wp_error($response)) {
            echo '<div class="notice notice-error"><p>Failed to fetch plugin list from the remote server.</p></div>';
            continue;
        }

        $body = wp_remote_retrieve_body($response);
        $plugins = json_decode($body, true);

        // Debugging: Print the JSON response
        error_log('Remote Plugin List Response: ' . print_r($body, true));

        if (json_last_error() !== JSON_ERROR_NONE) {
            echo '<div class="notice notice-error"><p>Failed to decode JSON response: ' . json_last_error_msg() . '</p></div>';
            continue;
        }

        if (empty($plugins) || !is_array($plugins)) {
            echo '<div class="notice notice-info"><p>No plugins found on the remote server or invalid response format.</p></div>';
            continue;
        }

        // Check for updates for the fetched plugins on the local server
        $updates = [];
        foreach ($plugins as $plugin) {
            if (!is_array($plugin) || !isset($plugin['plugin']) || !isset($plugin['version'])) {
                error_log('Invalid Plugin Data: ' . print_r($plugin, true));
                continue;
            }

            $plugin_slug = dirname($plugin['plugin']);
            $plugin_version = $plugin['version'];

            // Check if the plugin is from WordPress.org
            if (strpos($plugin_slug, '.') === false) {
                // Check if an update is available for the plugin
                $update_info = get_plugin_update_info($plugin_slug, $plugin_version);
                if ($update_info) {
                    $updates[] = $update_info;
                } else {
                    error_log('No update info found for plugin: ' . $plugin_slug);
                }
            } else {
                error_log('Skipping non-WordPress.org plugin: ' . $plugin_slug);
            }
        }

        if (!empty($updates)) {
            $all_updates[$site_url] = $updates;
        }
    }

    if (empty($all_updates)) {
        return ['message' => 'No plugin updates available.', 'type' => 'info'];
    }

    // Return the updates grouped by site
    return ['updates' => $all_updates, 'type' => 'success'];
}



function get_plugin_update_info($plugin_slug, $plugin_version) {
    if (strpos($plugin_slug, '/') !== false) {
        $plugin_slug = dirname($plugin_slug);
    }

    $request = [
        'action' => 'plugin_information',
        'request' => serialize((object)[
            'slug' => $plugin_slug,
            'fields' => [
                'short_description' => true,
                'sections' => false,
                'requires' => true,
                'rating' => false,
                'ratings' => false,
                'downloaded' => false,
                'last_updated' => true,
                'added' => false,
                'tags' => false,
                'compatibility' => false,
                'homepage' => true,
                'versions' => false,
                'donate_link' => false,
                'reviews' => false,
                'banners' => false,
                'icons' => false,
                'active_installs' => false,
                'group' => false,
                'contributors' => false,
            ],
        ]),
    ];

    $response = wp_remote_post('https://api.wordpress.org/plugins/info/1.0/', [
        'body' => $request,
    ]);

    if (is_wp_error($response)) {
        error_log('Error fetching plugin info for ' . $plugin_slug . ': ' . $response->get_error_message());
        return null;
    }

    $plugin_info = unserialize(wp_remote_retrieve_body($response));

    if ($plugin_info === false || is_wp_error($plugin_info)) {
        error_log('Invalid plugin info for ' . $plugin_slug);
        return null;
    }

    if (isset($plugin_info->version) && version_compare($plugin_info->version, $plugin_version, '>')) {
        return [
            'plugin' => $plugin_slug,
            'current_version' => $plugin_version,
            'new_version' => $plugin_info->version,
            'url' => $plugin_info->homepage ?? '',
            'package' => $plugin_info->download_link ?? '',
            'requires' => $plugin_info->requires ?? '',
            'tested' => $plugin_info->tested ?? '',
            'requires_php' => $plugin_info->requires_php ?? '',
            'last_updated' => $plugin_info->last_updated ?? '',
            'description' => $plugin_info->short_description ?? '',
        ];
    }
    return null;
}

